@@ -1,9 +1,5 @@
-// `mem::uninitialized` replaced with `mem::MaybeUninit`,
-// can't upgrade yet
-#![allow(deprecated)]
-
 use std::fmt::{self, Write};
-use std::mem;
+use std::mem::MaybeUninit;
 
 #[cfg(any(test, feature = "server", feature = "ffi"))]
 use bytes::Bytes;
@@ -12,6 +8,7 @@ use bytes::BytesMut;
 use http::header::ValueIter;
 use http::header::{self, Entry, HeaderName, HeaderValue};
 use http::{HeaderMap, Method, StatusCode, Version};
+use tracing::{debug, error, trace, trace_span, warn};
 
 use crate::body::DecodedLength;
 #[cfg(feature = "server")]
@@ -115,17 +112,18 @@ impl Http1Transaction for Server {
         // but we *never* read any of it until after httparse has assigned
         // values into it. By not zeroing out the stack memory, this saves
         // a good ~5% on pipeline benchmarks.
-        let mut headers_indices: [HeaderIndices; MAX_HEADERS] = unsafe { mem::uninitialized() };
+        let mut headers_indices: [MaybeUninit<HeaderIndices>; MAX_HEADERS] = unsafe {
+            // SAFETY: We can go safely from MaybeUninit array to array of MaybeUninit
+            MaybeUninit::uninit().assume_init()
+        };
         {
-            let mut headers: [httparse::Header<'_>; MAX_HEADERS] = unsafe { mem::uninitialized() };
-            trace!(
-                "Request.parse([Header; {}], [u8; {}])",
-                headers.len(),
-                buf.len()
-            );
-            let mut req = httparse::Request::new(&mut headers);
+            /* SAFETY: it is safe to go from MaybeUninit array to array of MaybeUninit */
+            let mut headers: [MaybeUninit<httparse::Header<'_>>; MAX_HEADERS] =
+                unsafe { MaybeUninit::uninit().assume_init() };
+            trace!(bytes = buf.len(), "Request.parse");
+            let mut req = httparse::Request::new(&mut []);
             let bytes = buf.as_ref();
-            match req.parse(bytes) {
+            match req.parse_with_uninit_headers(bytes, &mut headers) {
                 Ok(httparse::Status::Complete(parsed_len)) => {
                     trace!("Request.parse Complete({})", parsed_len);
                     len = parsed_len;
@@ -142,7 +140,6 @@ impl Http1Transaction for Server {
                         is_http_11 = false;
                         Version::HTTP_10
                     };
-                    trace!("headers: {:?}", &req.headers);
 
                     record_header_indices(bytes, &req.headers, &mut headers_indices)?;
                     headers_len = req.headers.len();
@@ -194,6 +191,8 @@ impl Http1Transaction for Server {
         headers.reserve(headers_len);
 
         for header in &headers_indices[..headers_len] {
+            // SAFETY: array is valid up to `headers_len`
+            let header = unsafe { &*header.as_ptr() };
             let name = header_name!(&slice[header.name.0..header.name.1]);
             let value = header_value!(slice.slice(header.value.0..header.value.1));
 
@@ -356,7 +355,7 @@ impl Http1Transaction for Server {
         }
 
         let orig_headers;
-        let extensions = mem::take(&mut msg.head.extensions);
+        let extensions = std::mem::take(&mut msg.head.extensions);
         let orig_headers = match extensions.get::<HeaderCaseMap>() {
             None if msg.title_case_headers => {
                 orig_headers = HeaderCaseMap::default();
@@ -867,18 +866,22 @@ impl Http1Transaction for Client {
         // Loop to skip information status code headers (100 Continue, etc).
         loop {
             // Unsafe: see comment in Server Http1Transaction, above.
-            let mut headers_indices: [HeaderIndices; MAX_HEADERS] = unsafe { mem::uninitialized() };
+            let mut headers_indices: [MaybeUninit<HeaderIndices>; MAX_HEADERS] = unsafe {
+                // SAFETY: We can go safely from MaybeUninit array to array of MaybeUninit
+                MaybeUninit::uninit().assume_init()
+            };
             let (len, status, reason, version, headers_len) = {
-                let mut headers: [httparse::Header<'_>; MAX_HEADERS] =
-                    unsafe { mem::uninitialized() };
-                trace!(
-                    "Response.parse([Header; {}], [u8; {}])",
-                    headers.len(),
-                    buf.len()
-                );
-                let mut res = httparse::Response::new(&mut headers);
+                // SAFETY: We can go safely from MaybeUninit array to array of MaybeUninit
+                let mut headers: [MaybeUninit<httparse::Header<'_>>; MAX_HEADERS] =
+                    unsafe { MaybeUninit::uninit().assume_init() };
+                trace!(bytes = buf.len(), "Response.parse");
+                let mut res = httparse::Response::new(&mut []);
                 let bytes = buf.as_ref();
-                match ctx.h1_parser_config.parse_response(&mut res, bytes) {
+                match ctx.h1_parser_config.parse_response_with_uninit_headers(
+                    &mut res,
+                    bytes,
+                    &mut headers,
+                ) {
                     Ok(httparse::Status::Complete(len)) => {
                         trace!("Response.parse Complete({})", len);
                         let status = StatusCode::from_u16(res.code.unwrap())?;
@@ -934,6 +937,8 @@ impl Http1Transaction for Client {
 
             headers.reserve(headers_len);
             for header in &headers_indices[..headers_len] {
+                // SAFETY: array is valid up to `headers_len`
+                let header = unsafe { &*header.as_ptr() };
                 let name = header_name!(&slice[header.name.0..header.name.1]);
                 let value = header_value!(slice.slice(header.value.0..header.value.1));
 
@@ -991,6 +996,13 @@ impl Http1Transaction for Client {
                 }));
             }
 
+            #[cfg(feature = "ffi")]
+            if head.subject.is_informational() {
+                if let Some(callback) = ctx.on_informational {
+                    callback.call(head.into_response(crate::Body::empty()));
+                }
+            }
+
             // Parsing a 1xx response could have consumed the buffer, check if
             // it is empty now...
             if buf.is_empty() {
@@ -1281,7 +1293,7 @@ struct HeaderIndices {
 fn record_header_indices(
     bytes: &[u8],
     headers: &[httparse::Header<'_>],
-    indices: &mut [HeaderIndices],
+    indices: &mut [MaybeUninit<HeaderIndices>],
 ) -> Result<(), crate::error::Parse> {
     let bytes_ptr = bytes.as_ptr() as usize;
 
@@ -1292,45 +1304,36 @@ fn record_header_indices(
         }
         let name_start = header.name.as_ptr() as usize - bytes_ptr;
         let name_end = name_start + header.name.len();
-        indices.name = (name_start, name_end);
         let value_start = header.value.as_ptr() as usize - bytes_ptr;
         let value_end = value_start + header.value.len();
-        indices.value = (value_start, value_end);
+
+        // FIXME(maybe_uninit_extra)
+        // FIXME(addr_of)
+        // Currently we don't have `ptr::addr_of_mut` in stable rust or
+        // MaybeUninit::write, so this is some way of assigning into a MaybeUninit
+        // safely
+        let new_header_indices = HeaderIndices {
+            name: (name_start, name_end),
+            value: (value_start, value_end),
+        };
+        *indices = MaybeUninit::new(new_header_indices);
     }
 
     Ok(())
 }
 
-// Write header names as title case. The header name is assumed to be ASCII,
-// therefore it is trivial to convert an ASCII character from lowercase to
-// uppercase. It is as simple as XORing the lowercase character byte with
-// space.
+// Write header names as title case. The header name is assumed to be ASCII.
 fn title_case(dst: &mut Vec<u8>, name: &[u8]) {
     dst.reserve(name.len());
 
-    let mut iter = name.iter();
-
-    // Uppercase the first character
-    if let Some(c) = iter.next() {
-        if *c >= b'a' && *c <= b'z' {
-            dst.push(*c ^ b' ');
-        } else {
-            dst.push(*c);
-        }
-    }
-
-    while let Some(c) = iter.next() {
-        dst.push(*c);
-
-        if *c == b'-' {
-            if let Some(c) = iter.next() {
-                if *c >= b'a' && *c <= b'z' {
-                    dst.push(*c ^ b' ');
-                } else {
-                    dst.push(*c);
-                }
-            }
+    // Ensure first character is uppercased
+    let mut prev = b'-';
+    for &(mut c) in name {
+        if prev == b'-' {
+            c.make_ascii_uppercase();
         }
+        dst.push(c);
+        prev = c;
     }
 }
 
@@ -1428,6 +1431,8 @@ mod tests {
                 preserve_header_case: false,
                 h09_responses: false,
                 #[cfg(feature = "ffi")]
+                on_informational: &mut None,
+                #[cfg(feature = "ffi")]
                 raw_headers: false,
             },
         )
@@ -1453,6 +1458,8 @@ mod tests {
             preserve_header_case: false,
             h09_responses: false,
             #[cfg(feature = "ffi")]
+            on_informational: &mut None,
+            #[cfg(feature = "ffi")]
             raw_headers: false,
         };
         let msg = Client::parse(&mut raw, ctx).unwrap().unwrap();
@@ -1473,6 +1480,8 @@ mod tests {
             preserve_header_case: false,
             h09_responses: false,
             #[cfg(feature = "ffi")]
+            on_informational: &mut None,
+            #[cfg(feature = "ffi")]
             raw_headers: false,
         };
         Server::parse(&mut raw, ctx).unwrap_err();
@@ -1491,6 +1500,8 @@ mod tests {
             preserve_header_case: false,
             h09_responses: true,
             #[cfg(feature = "ffi")]
+            on_informational: &mut None,
+            #[cfg(feature = "ffi")]
             raw_headers: false,
         };
         let msg = Client::parse(&mut raw, ctx).unwrap().unwrap();
@@ -1511,6 +1522,8 @@ mod tests {
             preserve_header_case: false,
             h09_responses: false,
             #[cfg(feature = "ffi")]
+            on_informational: &mut None,
+            #[cfg(feature = "ffi")]
             raw_headers: false,
         };
         Client::parse(&mut raw, ctx).unwrap_err();
@@ -1535,6 +1548,8 @@ mod tests {
             preserve_header_case: false,
             h09_responses: false,
             #[cfg(feature = "ffi")]
+            on_informational: &mut None,
+            #[cfg(feature = "ffi")]
             raw_headers: false,
         };
         let msg = Client::parse(&mut raw, ctx).unwrap().unwrap();
@@ -1556,6 +1571,8 @@ mod tests {
             preserve_header_case: false,
             h09_responses: false,
             #[cfg(feature = "ffi")]
+            on_informational: &mut None,
+            #[cfg(feature = "ffi")]
             raw_headers: false,
         };
         Client::parse(&mut raw, ctx).unwrap_err();
@@ -1572,6 +1589,8 @@ mod tests {
             preserve_header_case: true,
             h09_responses: false,
             #[cfg(feature = "ffi")]
+            on_informational: &mut None,
+            #[cfg(feature = "ffi")]
             raw_headers: false,
         };
         let parsed_message = Server::parse(&mut raw, ctx).unwrap().unwrap();
@@ -1609,6 +1628,8 @@ mod tests {
                     preserve_header_case: false,
                     h09_responses: false,
                     #[cfg(feature = "ffi")]
+                    on_informational: &mut None,
+                    #[cfg(feature = "ffi")]
                     raw_headers: false,
                 },
             )
@@ -1627,6 +1648,8 @@ mod tests {
                     preserve_header_case: false,
                     h09_responses: false,
                     #[cfg(feature = "ffi")]
+                    on_informational: &mut None,
+                    #[cfg(feature = "ffi")]
                     raw_headers: false,
                 },
             )
@@ -1854,6 +1877,8 @@ mod tests {
                     preserve_header_case: false,
                     h09_responses: false,
                     #[cfg(feature = "ffi")]
+                    on_informational: &mut None,
+                    #[cfg(feature = "ffi")]
                     raw_headers: false,
                 }
             )
@@ -1872,6 +1897,8 @@ mod tests {
                     preserve_header_case: false,
                     h09_responses: false,
                     #[cfg(feature = "ffi")]
+                    on_informational: &mut None,
+                    #[cfg(feature = "ffi")]
                     raw_headers: false,
                 },
             )
@@ -1890,6 +1917,8 @@ mod tests {
                     preserve_header_case: false,
                     h09_responses: false,
                     #[cfg(feature = "ffi")]
+                    on_informational: &mut None,
+                    #[cfg(feature = "ffi")]
                     raw_headers: false,
                 },
             )
@@ -2283,6 +2312,8 @@ mod tests {
             .insert("content-length", HeaderValue::from_static("10"));
         head.headers
             .insert("content-type", HeaderValue::from_static("application/json"));
+        head.headers
+            .insert("weird--header", HeaderValue::from_static(""));
 
         let mut vec = Vec::new();
         Server::encode(
@@ -2298,7 +2329,7 @@ mod tests {
         .unwrap();
 
         let expected_response =
-            b"HTTP/1.1 200 OK\r\nContent-Length: 10\r\nContent-Type: application/json\r\n";
+            b"HTTP/1.1 200 OK\r\nContent-Length: 10\r\nContent-Type: application/json\r\nWeird--Header: \r\n";
 
         assert_eq!(&vec[..expected_response.len()], &expected_response[..]);
     }
@@ -2383,6 +2414,8 @@ mod tests {
                 preserve_header_case: false,
                 h09_responses: false,
                 #[cfg(feature = "ffi")]
+                on_informational: &mut None,
+                #[cfg(feature = "ffi")]
                 raw_headers: false,
             },
         )
@@ -2465,6 +2498,8 @@ mod tests {
                     preserve_header_case: false,
                     h09_responses: false,
                     #[cfg(feature = "ffi")]
+                    on_informational: &mut None,
+                    #[cfg(feature = "ffi")]
                     raw_headers: false,
                 },
             )
@@ -2503,6 +2538,8 @@ mod tests {
                     preserve_header_case: false,
                     h09_responses: false,
                     #[cfg(feature = "ffi")]
+                    on_informational: &mut None,
+                    #[cfg(feature = "ffi")]
                     raw_headers: false,
                 },
             )
